"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processLogBuilder = exports.convertLevel = void 0;
const p_retry_1 = __importDefault(require("p-retry"));
const exit_hook_1 = __importDefault(require("exit-hook"));
const log_storage_1 = require("./log-storage");
// Define log sending limits
// https://docs.datadoghq.com/api/latest/logs/#send-logs
// Use 4.9 MB instead of 5 MB to determine when to stop batching
const LOGS_PAYLOAD_SIZE_LIMIT = 5138022;
// Use 0.95 MB instead of 1 MB to account for tags and other metadata
const LOG_SIZE_LIMIT = 996147;
const FORCE_SEND_MS = 3000;
const MAX_LOG_ITEMS = 995;
const convertLevel = (level) => {
    if (typeof level === 'string') {
        return level;
    }
    if (level >= 60) {
        return 'fatal';
    }
    if (level >= 50) {
        return 'error';
    }
    if (level >= 40) {
        return 'warning';
    }
    if (level >= 30) {
        return 'info';
    }
    if (level >= 20) {
        return 'debug';
    }
    return 'trace';
};
exports.convertLevel = convertLevel;
function sendLogs({ apiInstance, logsToSend, bucketName, options }) {
    (0, p_retry_1.default)(async () => {
        const params = {
            body: logsToSend,
            contentEncoding: 'gzip',
        };
        const result = await apiInstance.submitLog(params);
        if (options.onDebug) {
            options.onDebug(`(${bucketName}) Sending ${logsToSend.length} logs to Datadog completed`);
        }
        return result;
    }, { retries: options.retries ?? 5 }).catch((err) => {
        if (options.onError) {
            options.onError(err, logsToSend);
        }
    });
}
function processLogBuilder(options, apiInstance) {
    const logStorage = new log_storage_1.LogStorage();
    let timer = null;
    if (!options.sendImmediate) {
        if (options.onDebug) {
            options.onDebug(`Configured to send logs every ${options.sendIntervalMs || FORCE_SEND_MS}ms`);
        }
        timer = setInterval(() => {
            const logCount = logStorage.getLogCount();
            const currentBucket = logStorage.currentBucket;
            if (logCount > 0) {
                if (options.onDebug) {
                    options.onDebug(`(${currentBucket}) Sending ${logCount} logs to Datadog on timer`);
                }
                // ...logItems is so if we clear logItems in another run, we won't lose these logs
                sendLogs({
                    apiInstance: apiInstance,
                    logsToSend: logStorage.finishLogBatch(),
                    bucketName: currentBucket,
                    options: options,
                });
            }
        }, options.sendIntervalMs || FORCE_SEND_MS);
    }
    if (options.onDebug) {
        options.onDebug('Configuring exit hook');
    }
    (0, exit_hook_1.default)(() => {
        if (logStorage.getLogCount() > 0) {
            // Attempt to send logs on an exit call
            if (options.onDebug) {
                options.onDebug(`Shutdown detected. Attempting to send remaining logs to Datadog`);
            }
            if (timer) {
                clearInterval(timer);
            }
            const currentBucket = logStorage.currentBucket;
            sendLogs({
                apiInstance: apiInstance,
                logsToSend: logStorage.finishLogBatch(),
                bucketName: currentBucket,
                options: options,
            });
        }
    });
    if (options.onInit) {
        options.onInit();
    }
    return async function processLogs(source) {
        for await (const obj of source) {
            if (!obj) {
                if (options.onDebug) {
                    options.onDebug('Log source object is empty');
                }
                return;
            }
            // Datadog uses the date field for timestamp
            obj.date = obj.time;
            delete obj.time;
            const logItem = {
                message: JSON.stringify({
                    ...obj,
                    level: (0, exports.convertLevel)(obj.level),
                }),
            };
            if (options.ddsource) {
                logItem.ddsource = options.ddsource;
            }
            if (options.ddtags) {
                logItem.ddtags = options.ddtags;
            }
            if (options.service) {
                logItem.service = options.service;
            }
            if (obj.hostname) {
                logItem.hostname = obj.hostname;
            }
            const logEntryLength = logItem.message.length +
                (logItem?.ddsource?.length || 0) +
                (logItem?.ddtags?.length || 0) +
                (logItem?.hostname?.length || 0) +
                (logItem?.service?.length || 0);
            if (logEntryLength > LOG_SIZE_LIMIT) {
                if (options.onError) {
                    options.onError(new Error(`Log entry exceeds size limit of ${LOG_SIZE_LIMIT} bytes: ${logEntryLength}`), [
                        logItem,
                    ]);
                }
            }
            if (options.sendImmediate) {
                if (options.onDebug) {
                    options.onDebug(`(send-immediate) Sending log to Datadog`);
                }
                // Don't go through the batch logger if sendImmediate is enabled
                sendLogs({
                    apiInstance: apiInstance,
                    logsToSend: [logItem],
                    bucketName: 'send-immediate',
                    options: options,
                });
                continue;
            }
            logStorage.addLog(logItem, logEntryLength);
            const logCount = logStorage.getLogCount();
            const shouldSend = logStorage.getLogBucketByteSize() > LOGS_PAYLOAD_SIZE_LIMIT || logCount > MAX_LOG_ITEMS;
            if (shouldSend) {
                // Get the bucket name into a variable as finishLogBatch() generates a new one
                const currentBucket = logStorage.currentBucket;
                if (options.onDebug) {
                    options.onDebug(`(${currentBucket}) Sending ${logCount} logs to Datadog`);
                }
                sendLogs({
                    apiInstance: apiInstance,
                    logsToSend: logStorage.finishLogBatch(),
                    bucketName: currentBucket,
                    options: options,
                });
            }
        }
    };
}
exports.processLogBuilder = processLogBuilder;
//# sourceMappingURL=process-logs.js.map